/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;


/**
 * <p>
 * This abstract implementation of the NodeMatcher interface provides title and url type services
 *  to subclasses.  This implementation will provide
 *  a title and url holder variable and a getter methods.
 * </p>
 * <p><b>Implementation notes: </b> The developer should strongly encourage that all implementations
 *  of the NodeMatcher inherit from
 * this abstract base.  This abstract base currently, only provides title/url services but may
 *  provide additional services in the future.
 * </p>
 * <p>Thread safety:
 * This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public abstract class AbstractNodeMatcher implements NodeMatcher {
    /**
     * <p>Represents the default title that a node can take if it matches this node matcher.
     * This variable is set by the constructor, is immutable and can never be null nor an empty
     *  (trim'd) string,  The title will be referenced in the getTitle method.</p>
     */
    private final String title;

    /**
     * <p>Represents the default url that can be used if not contextual information can be used.
     *  This variable is set by the constructor, is immutable and can never be null nor an empty
     *  (trim'd) string,  The url will be referenced in the getTitle method.</p>
     */
    private final String url;

    /**
     * <p>Constructs the abstract base from the given title and url.</p>
     *
     *
     * @param title A non-null, non-empty (trim'd) title to use
     * @param url A non-null, non-empty (trim'd) url to use
     * @throws IllegalArgumentException if either arg is null or an empty (trim'd) string
     */
    protected AbstractNodeMatcher(String title, String url) {
        if (title == null) {
            throw new IllegalArgumentException("title cannot be null.");
        }

        if (title.trim().length() == 0) {
            throw new IllegalArgumentException("title cannot be empty string.");
        }

        if (url == null) {
            throw new IllegalArgumentException("url cannot be null.");
        }

        if (url.trim().length() == 0) {
            throw new IllegalArgumentException("url cannot be empty string.");
        }

        this.title = title;
        this.url = url;
    }

    /**
     * <p>Simply the getter method for the default title.</p>
     *
     *
     * @return A non-null, non-empty (trim'd) string representing the title
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>Simply the getter method for the url.</p>
     *
     *
     * @return A non-null, non-empty (trim'd) string representing the url to us
     */
    public String getUrl() {
        return url;
    }
}
