/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.AbstractNodeMatcher;

import java.util.regex.Pattern;


/**
 * <p>This implementation of a NodeMatcher will provide node matching services given a specific
 * regex pattern. This class will return true when the passed node's url matches the specified
 * pattern.</p>
 * <p>Thread Safety:  This class is thread safe by being immutable.</p>
 *
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class RegexNodeMatcher extends AbstractNodeMatcher {
    /**
     * <p>Represents the pattern that will be used to determine if the node matches.  This
     * variable is set by the constructor, is immutable and can never be null or an empty
     *  (trim'd) string, The pattern will be returned in the getPathPattern getter method.</p>
     */
    private final String nodePattern;

    /**
     * <p>Represents the compiled version of the nodePattern.  This variable is set
     * by the constructor, is immutable and can never be null,  The pattern will be used
     * in the matches method.</p>
     */
    private final Pattern pattern;

    /**
     * <p>Constructs the RegexNodeMatcher from the given title, url and pattern.</p>
     *
     *
     * @param title A non-null, non-empty (trim'd) string describing the default title of the node
     * @param url A non-null, non-empty (trim'd) string representing the url to us
     * @param nodePattern A non-null, non-empty (trim'd) string representing the node pattern to apply
     * @throws IllegalArgumentException if any argument is a null or an empty (trim'd) string
     */
    public RegexNodeMatcher(String title, String url, String nodePattern) {
        super(title, url);
        validateNotNullOrEmpty(title, "title");
        validateNotNullOrEmpty(url, "url");
        validateNotNullOrEmpty(nodePattern, "nodePattern");

        this.nodePattern = nodePattern;
        pattern = Pattern.compile(nodePattern);
    }

    /**
     * <p>Returns true if the node's url matches the specified pattern.</p>
     *
     *
     * @param node A non-null node to test
     * @return True if the node matches the regex pattern, false otherwise
     */
    public boolean matches(BreadCrumbNode node) {
        return pattern.matcher(node.getUrl()).matches();
    }

    /**
     * <p>Simply the getter method for the node pattern string.</p>
     *
     *
     * @return A non-null, non-empty (trim'd) string representing the regex pattern used to
     *  match against nodes
     */
    public String getNodePattern() {
        return this.nodePattern;
    }

    /**
     * <p>Returns true if the given object matches this node matcher.</p>
     *
     *
     * @param o A possibly null object to test for equality
     * @return True if the object is of type RegexNodeMatcher with a matching nodePattern/title/url
     */
    public boolean equals(Object o) {
        if ((o == null) || (o instanceof RegexNodeMatcher)) {
            return false;
        }

        RegexNodeMatcher matcher = (RegexNodeMatcher) o;

        return this.nodePattern.equals(matcher.nodePattern) && this.pattern.equals(matcher.pattern)
        && this.getTitle().equals(matcher.getTitle()) && this.getUrl().equals(matcher.getUrl());
    }

    /**
     * <p>Returns the hash code for this object.</p>
     *
     *
     * @return The hashcode of the object
     */
    public int hashCode() {
        final int in = 29;
        return (nodePattern.hashCode() + in) ^ (getTitle().hashCode() + in) ^ getUrl().hashCode();
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> and
     *  empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    private void validateNotNullOrEmpty(String value, String name) {
        if (value == null) {
            throw new IllegalArgumentException(name + " cannot be null.");
        }

        if (value.trim().length() == 0) {
            throw new IllegalArgumentException(name + " cannot be empty string.");
        }
    }
}
